@@ -18,13 +18,13 @@ require 'google/apis'
 require 'google/apis/core/version'
 require 'google/apis/core/api_command'
 require 'google/apis/core/batch'
+require 'google/apis/core/faraday_integration'
 require 'google/apis/core/upload'
 require 'google/apis/core/storage_upload'
 require 'google/apis/core/download'
 require 'google/apis/core/storage_download'
 require 'google/apis/options'
 require 'googleauth'
-require 'httpclient'
 
 module Google
   module Apis
@@ -93,11 +93,49 @@ module Google
       # Base service for all APIs. Not to be used directly.
       #
       class BaseService
+        ##
+        # A substitution string for the universe domain in an endpoint template
+        # @return [String]
+        #
+        ENDPOINT_SUBSTITUTION = "$UNIVERSE_DOMAIN$".freeze
+
         include Logging
 
+        # Universe domain
+        # @return [String]
+        attr_reader :universe_domain
+
+        # Set the universe domain.
+        # If the root URL was set with a universe domain substitution, it is
+        # updated to reflect the new universe domain.
+        #
+        # @param new_ud [String,nil] The new universe domain, or nil to use the Google Default Universe
+        def universe_domain= new_ud
+          new_ud ||= ENV["GOOGLE_CLOUD_UNIVERSE_DOMAIN"] || "googleapis.com"
+          if @root_url_template
+            @root_url = @root_url_template.gsub ENDPOINT_SUBSTITUTION, new_ud
+          end
+          @universe_domain = new_ud
+        end
+
         # Root URL (host/port) for the API
-        # @return [Addressable::URI]
-        attr_accessor :root_url
+        # @return [Addressable::URI, String]
+        attr_reader :root_url
+
+        # Set the root URL.
+        # If the given url includes a universe domain substitution, it is
+        # resolved in the current universe domain
+        #
+        # @param url_or_template [Addressable::URI, String] The URL, which can include a universe domain substitution
+        def root_url= url_or_template
+          if url_or_template.is_a?(String) && url_or_template.include?(ENDPOINT_SUBSTITUTION)
+            @root_url_template = url_or_template
+            @root_url = url_or_template.gsub ENDPOINT_SUBSTITUTION, universe_domain
+          else
+            @root_url_template = nil
+            @root_url = url_or_template
+          end
+        end
 
         # Additional path prefix for all API methods
         # @return [Addressable::URI]
@@ -111,9 +149,9 @@ module Google
         # @return [Addressable::URI]
         attr_accessor :batch_path
 
-        # HTTP client
-        # @return [HTTPClient]
-        attr_accessor :client
+        # Faraday HTTP connection
+        # @return [Faraday::Connection]
+        attr_writer :client
 
         # General settings
         # @return [Google::Apis::ClientOptions]
@@ -136,7 +174,9 @@ module Google
         # @param [String,Addressable::URI] base_path
         #   Additional path prefix for all API methods
         # @api private
-        def initialize(root_url, base_path, client_name: nil, client_version: nil)
+        def initialize(root_url, base_path, client_name: nil, client_version: nil, universe_domain: nil)
+          @root_url_template = nil
+          self.universe_domain = universe_domain
           self.root_url = root_url
           self.base_path = base_path
           self.client_name = client_name || 'google-api-ruby-client'
@@ -223,8 +263,8 @@ module Google
           batch_command.execute(client)
         end
 
-        # Get the current HTTP client
-        # @return [HTTPClient]
+        # Get the current HTTP connection
+        # @return [Faraday::Connection]
         def client
           @client ||= new_client
         end
@@ -279,7 +319,7 @@ module Google
         #   Name of the field in the result containing the items. Defaults to :items
         # @param [Boolean] cache
         #  True (default) if results should be cached so multiple iterations can be used.
-        # @return [Enumerble]
+        # @return [Enumerable]
         # @yield [token, service]
         #   Current page token & service instance
         # @yieldparam [String] token
@@ -296,6 +336,50 @@ module Google
           return PagedResults.new(self, max: max, items: items, cache: cache, response_page_token: response_page_token, &block)
         end
 
+        # Verify that the universe domain setting matches the universe domain
+        # in the credentials, if present.
+        #
+        # @raise [Google::Apis::UniverseDomainError] if there is a mismatch
+        def verify_universe_domain!
+          auth = authorization
+          auth_universe_domain = auth.universe_domain if auth.respond_to? :universe_domain
+          if auth_universe_domain && auth_universe_domain != universe_domain
+            raise UniverseDomainError,
+                  "Universe domain is #{universe_domain} but credentials are in #{auth_universe_domain}"
+          end
+          true
+        end
+
+        # Restarts An interrupted Resumable upload
+        # @param [String] bucket
+        #   Name of the bucket where the upload is being performed.
+        # @param [IO, String] upload_source
+        #   IO stream or filename containing content to upload
+        # @param [IO, String] upload_id
+        #   unique id generated for an ongoing upload
+
+        def restart_resumable_upload(bucket, upload_source, upload_id, options: nil)
+          command = make_storage_upload_command(:put, 'b/{bucket}/o', options)
+          command.upload_source = upload_source
+          command.upload_id = upload_id
+          command.params['bucket'] = bucket unless bucket.nil?
+          execute_or_queue_command(command)
+        end
+
+        # Deletes An interrupted Resumable upload
+        # @param [String] bucket
+        #   Name of the bucket where the upload is being performed.
+        # @param [IO, String] upload_id
+        #   unique id generated for an ongoing upload
+
+        def delete_resumable_upload(bucket, upload_id, options: nil)
+          command = make_storage_upload_command(:delete, 'b/{bucket}/o', options)
+          command.upload_id = upload_id
+          command.params['bucket'] = bucket unless bucket.nil?
+          command.delete_upload = true
+          execute_or_queue_command(command)
+        end
+
         protected
 
         # Create a new upload command.
@@ -308,6 +392,7 @@ module Google
         #  Request-specific options
         # @return [Google::Apis::Core::UploadCommand]
         def make_upload_command(method, path, options)
+          verify_universe_domain!
           template = Addressable::Template.new(root_url + upload_path + path)
           if batch?
             command = MultipartUploadCommand.new(method, template, client_version: client_version)
@@ -332,6 +417,7 @@ module Google
         #  Request-specific options
         # @return [Google::Apis::Core::StorageUploadCommand]
         def make_storage_upload_command(method, path, options)
+          verify_universe_domain!
           template = Addressable::Template.new(root_url + upload_path + path)
           command = StorageUploadCommand.new(method, template, client_version: client_version)
           command.options = request_options.merge(options)
@@ -349,6 +435,7 @@ module Google
         #  Request-specific options
         # @return [Google::Apis::Core::DownloadCommand]
         def make_download_command(method, path, options)
+          verify_universe_domain!
           template = Addressable::Template.new(root_url + base_path + path)
           command = DownloadCommand.new(method, template, client_version: client_version)
           command.options = request_options.merge(options)
@@ -368,6 +455,7 @@ module Google
         #  Request-specific options
         # @return [Google::Apis::Core::StorageDownloadCommand]
         def make_storage_download_command(method, path, options)
+          verify_universe_domain!
           template = Addressable::Template.new(root_url + base_path + path)
           command = StorageDownloadCommand.new(method, template, client_version: client_version)
           command.options = request_options.merge(options)
@@ -386,6 +474,7 @@ module Google
         #  Request-specific options
         # @return [Google::Apis::Core::DownloadCommand]
         def make_simple_command(method, path, options)
+          verify_universe_domain!
           full_path =
             if path.start_with? "/"
               path[1..-1]
@@ -453,39 +542,22 @@ module Google
           Thread.current[:google_api_batch_service] = nil
         end
 
-        # Create a new HTTP client
-        # @return [HTTPClient]
+        # Create a new HTTP connection
+        # @return [Faraday::Connection]
         def new_client
-          client = ::HTTPClient.new
+          options = {}
+          request_options = {params_encoder: Faraday::FlatParamsEncoder}
+          options[:proxy] = {uri: client_options.proxy_url} if client_options.proxy_url
+          request_options[:open_timeout] = client_options.open_timeout_sec if client_options.open_timeout_sec
+          request_options[:read_timeout] = client_options.read_timeout_sec if client_options.read_timeout_sec
+          request_options[:write_timeout] = client_options.send_timeout_sec if client_options.send_timeout_sec
+          options[:request] = request_options unless request_options.empty?
+          options[:headers] = { 'User-Agent' => user_agent }
 
-          if client_options.transparent_gzip_decompression
-            client.transparent_gzip_decompression = client_options.transparent_gzip_decompression
+          Faraday.new options do |faraday|
+            faraday.response :logger, Google::Apis.logger if client_options.log_http_requests
+            faraday.response :follow_redirects_google_apis_core, limit: 5
           end
-          
-          client.proxy = client_options.proxy_url if client_options.proxy_url
-
-          if client_options.open_timeout_sec
-            client.connect_timeout = client_options.open_timeout_sec
-          end
-
-          if client_options.read_timeout_sec
-            client.receive_timeout = client_options.read_timeout_sec
-          end
-
-          if client_options.send_timeout_sec
-            client.send_timeout = client_options.send_timeout_sec
-          end
-
-          client.follow_redirect_count = 5
-          client.default_header = { 'User-Agent' => user_agent }
-
-          client.debug_dev = logger if client_options.log_http_requests
-
-          # Make HttpClient use system default root CA path
-          # https://github.com/nahi/httpclient/issues/445
-          client.ssl_config.clear_cert_store
-          client.ssl_config.cert_store.set_default_paths
-          client
         end
 
 
