@@ -41,7 +41,8 @@ module Google
       :quota_project,
       :query,
       :add_invocation_id_header,
-      :upload_chunk_size)
+      :upload_chunk_size
+    )
 
     # General client options
     class ClientOptions
@@ -60,7 +61,7 @@ module Google
       # @!attribute [rw] read_timeout_sec
       #   @return [Integer] How long, in seconds, before receiving data times out
       # @!attribute [rw] transparent_gzip_decompression
-      # @return [Boolean] True if gzip compression needs to be enabled
+      #   @return [Boolean] DEPRECATED. Gzip decompression is now always on.
       # Get the default options
       # @return [Google::Apis::ClientOptions]
       def self.default
